/**
 * Title :        XLogo
 * Description :  XLogo is an interpreter for the Logo 
 * 						programming language
 * @author Loïc Le Coq
 */
package xlogo.kernel;

import java.util.HashMap;
import java.util.Stack;
import java.util.Set;
import java.util.Iterator;
import xlogo.Logo;
import xlogo.Application;
import xlogo.kernel.gui.GuiMap;

import java.io.BufferedReader;
import java.io.StringReader;
import java.io.IOException;
public class Workspace {
	protected  HashMap globale;  // HashMap with the name of all defined variables
								// and their values
	private Stack procedureList; // stack with all procedures defined by the user

	// For all Gui Object (Buttons, ComboBoxes...)
	private GuiMap gm;

	
	public Workspace(Application app) {
		globale= new HashMap();
		procedureList=new Stack();
		gm=new GuiMap(app);
	}
	public void deleteAllVariables(){
		globale = new HashMap();
	}
	public void deleteAllProcedures(){
		for (int i = procedureList.size()-1; i > -1; i--) {
			  Procedure procedure = getProcedure(i);
			  if (procedure.affichable == true)
				deleteProcedure(i);
			}
	}
	public Procedure procedureListPeek(){
		return  (Procedure)(procedureList.peek());
	}
	public void procedureListPop(){
		procedureList.pop();
	}
	public void procedureListPush(Procedure pr){
		procedureList.push(pr);
	}
	public void setProcedureList(int id, Procedure pr){
		procedureList.set(id,pr);
	}
	public Procedure getProcedure(int index){
		return (Procedure)(procedureList.get(index)); 	
	}
	public int getNumberOfProcedure(){
		return procedureList.size();
	}
	public void deleteProcedure(int index){
		procedureList.remove(index);
	}

	public void deleteVariable(String st){
		boolean b = globale.containsKey(st);
		if (b) {
			globale.remove(st);
		}
	}
	public String toString(){
		StringBuffer sb=new StringBuffer();
		Set set=globale.keySet();
		Iterator it=set.iterator();
		while(it.hasNext()){
			String key=it.next().toString();
			sb.append("-");
			sb.append(key);
			sb.append("\n");
			sb.append(globale.get(key));
			sb.append("\n");
		}
		for(int i=0;i<getNumberOfProcedure();i++){
	        Procedure procedure =getProcedure(i);
	        sb.append(Logo.messages.getString("pour") + " " + procedure.name);
	        for (int j = 0; j < procedure.nbparametre; j++) {
	        	sb.append(" :" +procedure.variable.get(j).toString());
	        }
	        sb.append("\n");
	        sb.append(procedure.instruction);
	        sb.append(Logo.messages.getString("fin"));
	        sb.append("\n\n");
	      }
		return (new String(sb));
	}
	public void setWorkspace(Application app,String wp){
		globale= new HashMap();
		procedureList=new Stack();

		StringReader sr=new StringReader(wp);
		BufferedReader bfr=new BufferedReader(sr);
		try{
			String input="";
			while((input=bfr.readLine())!=null){
				if (!input.startsWith(Logo.messages.getString("pour"))){
						globale.put(input.substring(1),bfr.readLine());
					}
				else break;
			}
			StringBuffer sb=new StringBuffer();
			if (null!=input){
				sb.append(input);
				sb.append("\n");
			}
			while((input=bfr.readLine())!=null){
				sb.append(input);
				sb.append("\n");
			}
			bfr.close();
			sr.close();
			try{
				app.editeur.setAffichable(false);
				app.editeur.setEditorText(new String(sb));
				app.editeur.analyseprocedure();
			}
			catch(Exception e){
			}
			
		}
		catch(IOException e){
		}
	}
	public GuiMap getGuiMap(){
		return gm;
	}
}
